function [c,ceq,dc,dceq] = VARconstraint(Y,R,p,h,theta,Z,atilde)
%_____________________________________________________________________________________________________________________
% Computes the constraint that the judgmental forecast is equal to the
% model forecast to implement the Wald test
%
%   PARAMETERS:
%   Y is an n-vector with the data of the dependent variable
%   R is the matrix of constraints
%   p is maximum number of lags allowed in the general VAR(p) model
%   h is the number of step ahead forecasts
%   theta is the vector of QMLE
%   Z is the matrix containing all the lags of the VAR(p) model
%   atilde is the judgmental decision
%
%   c and dc are empty matrices
%   ceq = mean(Yf)-atilde is the constraint that the average forecast is equal to the judgmental decision
%   dceq is the derivative of constraint
%_____________________________________________________________________________________________________________________
%
[g,n] = size(Y);   % g is the number of dependent variables
                   % n is the length of time series
Z = Z(:,n+1);      % Select the last column of Z for forecasting
J = sparse([zeros(g,1), eye(g), zeros(g,g*(p-1))]);
C = reshape(sparse(R)*theta, g, 1+g*p);
Cbar = sparse([1,                zeros(1,g*p);
               C;
               zeros(g*(p-1),1), eye(g*(p-1)), zeros(g*(p-1),g)]);
e1 = zeros(1,g); e1(1)=1; e1 = sparse(e1);
dC = zeros(g*h,g*(1+g*p));
Yf = zeros(h,1);
CCbar = zeros((h+1)*(1+g*p),1+g*p);
JCbar = zeros(g*(h+1),1+g*p);
JCbarJ = zeros(h*g,g);
CCbar(1:1+g*p,:) = eye(1+g*p);
JCbar(1:g,:) = J;
JCbarJ(1:g,:) = J*J';
for i = 1:h
    CCbar(i*(1+g*p)+1:(i+1)*(1+g*p),:) = Cbar^i;
    JCbar(i*g+1:g*(i+1),:) = J*CCbar(i*(1+g*p)+1:(i+1)*(1+g*p),:);
    JCbarJ(i*g+1:(i+1)*g,:) = JCbar(i*g+1:(i+1)*g,:)*J';
    Yf(i) = e1*JCbar(i*g+1:(i+1)*g,:)*Z; % Forecast of Y
    if nargout > 1 % Compute elements for VCf estimation
        for j = 0:i-1
            dC((i-1)*g+1:i*g,:) = dC((i-1)*g+1:i*g,:) + kron(Z'*CCbar((i-1-j)*(1+g*p)+1:(i-j)*(1+g*p),:)',JCbarJ(j*g+1:(j+1)*g,:));
        end
    end
end
c = [];
ceq = mean(Yf)-atilde; % Constraint
% Compute the derivative of the constraint
if nargout > 2
    dy = kron(ones(1,h),sparse(eye(g)))*dC/h;
    dc = [];
    dceq = e1*dy*R;
    dceq = dceq'; % Transpose to fit the constrained minimization algorithm
end